local addon = LFGTimer;
local L = LibStub("AceLocale-3.0"):GetLocale("LFGTimer");
local lsm = LibStub("LibSharedMedia-3.0");

if not addon then
	return;
end

lsm:Register("Background", "Blizzard Chat Bubble", "Interface\\Tooltips\\ChatBubble-Background");

local db; -- set after :InitConfig
local defaults = {
	profile = {
		LFGStartTime = 0,
		Frame = {
			Show = true,
			Option = "custom",
			Background = {
				Texture = "Blizzard Chat Bubble",
				Color = {
					r = 0,
					g = 0,
					b = 0,
					a = 1,
				},
				Tile = false,
				TileSize = 32,
			},
			Border = {
				Texture = "Blizzard Chat Bubble",
				Color = {
					r = 1,
					g = 1,
					b = 1,
					a = 1,
				},
				EdgeSize = 24,
			},
			Text = {
				Font = "Friz Quadrata TT",
				Outlint = "",
				Color = {
					r = 1,
					g = 1,
					b = 1,
					a = 1,
				},
				Size = 12,
			},
			Position = {
				AnchorAtSelf = "CENTER",
				AnchoredTo = "UIParent",
				AnchorAtOther = "CENTER",
				AnchorXOffset = 0,
				AnchorYOffset = 0,
			},
		},
	},
};

function addon:InitConfig()
	self.db = LibStub("AceDB-3.0"):New("LFGTimerDB", defaults);
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged");
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged");
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged");
	
	db = self.db.profile;
	
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("LFGTimer", self.Options)
	
	local lib = LibStub("AceConfigDialog-3.0");
	self.OptionsFrames = {};
	self.OptionsFrames.Main = lib:AddToBlizOptions("LFGTimer", "LFG Timer", nil, "General");
	self.OptionsFrames.Frame = lib:AddToBlizOptions("LFGTimer", L["Frame"], "LFG Timer", "Frame");
	self.Options.args.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
	self.OptionsFrames.Profiles = lib:AddToBlizOptions("LFGTimer", L["Profiles"], "LFG Timer", "Profiles");
end

function addon:OnProfileChanged(event, database)
	db = database.profile;
	self:UpdateFrameDB(db);
	self:UpdateMainDB(db);
end

addon.Options = {
	type = "group",
	order = 1,
	name = "LFG Timer",
	get = function(i)
		return db[ i[#i] ];
	end,
	set = function(i, v)
		db[ i[#i] ] = v;
	end,
	args = {
		General = {
			type = "group",
			order = 1,
			guiInline = true,
			name = L["General"],
			desc = L["General usage options"],
			args = {
				intro = {
					type = "description",
					name = L["LFGTimerAbout"],
					order = 1,
				},
			},
		},
		Frame = {
			type = "group",
			order = 2,
			guiInline = true,
			name = L["Frame"],
			desc = L["Floatin frame options"],
			get = function(i)
				return db.Frame[ i[#i] ];
			end,
			set = function(i, v)
				db.Frame[ i[#i] ] = v;
				addon:UpdateFrameTextures();
			end,
			hidden = function()
				--[[
					hide when no frame is shown or the frame shown is not the custom frame
				--]]
				return not db.Frame.Show or db.Frame.Option ~= "custom";
			end,
			args = {
				Show = {
					type = "toggle",
					order = 1,
					name = L["Show"],
					desc = L["Show the floating frame while in an LFG queue"],
					set = function(i, v)
						db.Frame.Show = v;
						addon:UpdateFrameShown();
					end,
					width = "double",
					hidden = false,
				},
				Option = {
					type = "select",
					order = 2,
					name = L["Frame to show"],
					desc = L["Choose which frame to show"],
					values = {
						["clone"] = L["Search status clone"],
						["custom"] = L["Custom frame"],
					},
					set = function(i, v)
						db.Frame.Option = v;
						addon:UpdateFrameShown();
					end,
					hidden = function()
						return not db.Frame.Show;
					end,
				},
				Background = {
					type = "group",
					order = 3,
					guiInline = true,
					name = L["Frame background options"],
					get = function(i)
						return db.Frame.Background[ i[#i] ];
					end,
					set = function(i, v)
						db.Frame.Background[ i[#i] ] = v;
						addon:UpdateFrameTextures(); -- all of the options here affect the floating frame
					end,
					args = {
						Texture = {
							type = "select",
							dialogControl = "LSM30_Background",
							order = 1,
							name = L["Background texture"],
							desc = L["Texture to use for the frame's background"],
							values = AceGUIWidgetLSMlists.background,
						},
						Color = {
							type = "color",
							order = 2,
							name = L["Background color"],
							desc = L["Frame's background color"],
							hasAlpha = true,
							get = function(i)
								local t = db.Frame.Background.Color;
								return t.r, t.g, t.b, t.a;
							end,
							set = function(i, ...) -- red, green, blue, alpha
								local t = db.Frame.Background.Color;
								t.r, t.g, t.b, t.a = ...;
								addon:UpdateFrameTextures();
							end,
						},
						Tile = {
							type = "toggle",
							order = 5,
							name = L["Tile background"],
							desc = L["Tile the background texture"],
						},
						TileSize = {
							type = "range",
							order = 6,
							name = L["Background tile size"],
							desc = L["The size used to tile the background texture"],
							min = 16, 
							max = 256,
							step = 1,
							disabled = function()
								return not db.Frame.Background.Tile;
							end,
						},
					},
				},
				Border = {
					type = "group",
					order = 4,
					guiInline = true,
					name = L["Frame border options"],
					get = function(i)
						return db.Frame.Border[ i[#i] ];
					end,
					set = function(i, v)
						db.Frame.Border[ i[#i] ] = v;
						addon:UpdateFrameTextures();
					end,
					args = {
						Texture = {
							type = "select",
							dialogControl = "LSM30_Border",
							order = 1,
							name = L["Border texture"],
							desc = L["Texture to use for the frame's border"],
							values = AceGUIWidgetLSMlists.border,
						},
						Color = {
							type = "color",
							order = 2,
							name = L["Border color"],
							desc = L["Frame's border color"],
							hasAlpha = true,
							get = function(i)
								local t = db.Frame.Border.Color;
								return t.r, t.g, t.b, t.a;
							end,
							set = function(i, ...) -- red, green, blue, alpha
								local t = db.Frame.Border.Color;
								t.r, t.g, t.b, t.a = ...
								addon:UpdateFrameTextures();
							end,
						},
						EdgeSize = {
							type = "range",
							order = 3,
							name = L["Border thickness"],
							desc = L["The thickness of the border"],
							min = 1,
							max = 32,
							step = 1,
						},
					},
				},
				Text = {
					type = "group",
					order = 5,
					guiInline = true,
					name = L["Text options"],
					get = function(i)
						return db.Frame.Text[ i[#i] ];
					end,
					set = function(i, v)
						db.Frame.Text[ i[#i] ] = v;
						addon:UpdateFrameFonts();
					end,
					args = {
						Font = {
							type = "select",
							dialogControl = "LSM30_Font",
							order = 1,
							name = L["Font"],
							desc = L["Font used to display text"],
							values = AceGUIWidgetLSMlists.font,
						},
						Outline = {
							type = "select",
							order = 2,
							name = L["Font outline"],
							desc = L["Outline text in the frame"],
							values = {
								[""] = L["None"],
								["OUTLINE"] = L["Outline"],
								["THICKOUTLINE"] = L["Thick outline"],
							},
						},
						Color = {
							type = "color",
							order = 3,
							name = L["Font color"],
							desc = L["Color of the font"],
							hasAlpha = true,
							get = function(i)
								local t = db.Frame.Text.Color;
								return t.r, t.g, t.b, t.a;
							end,
							set = function(i, ...) -- red, green, blue, alpha
								local t = db.Frame.Text.Color;
								t.r, t.g, t.b, t.a = ...;
							end
						},
						Size = {
							type = "range",
							order = 4,
							name = L["Font size"],
							desc = L["Set the font size of the text"],
							min = 4,
							max = 30,
							step = 1,
						},
					},
				},
			},
		},
	},
};
