if not io then
	print("Used to build locale data entries from various files for the LFG Timer addon"); -- in case some DIP#### loads this in WoW, which will likely be me in a few weeks/days
	error("Do not run LocaleWatcher.lua from inside WoW.");
end

--[[
	Extract data from a file based on a pattern. Data is extracted line-by-line.
	@param file file to read data from
	@param pat pattern used to extract data from the file
	@param t table used if appending data to an existing data set
	@returns a table that contains a list of matched data as keys of the returned table.
--]]
function extractData(file, pat, t)
	local f = assert(io.open(file, 'r'), ("Loading %s"):format(file));
	local ret = t or {}; -- return table
	local line;
	local func = function(d)
		ret[d] = true;
	end
	while true do
		line = f:read("*line");
		if not line then -- end of the file
			break;
		end
		string.gsub(line, pat, func);
	end
	f:close();
	return ret;
end

--[[
	@param file file stream to output data to
	@param arr key-indexed table to be written to `file`
	@returns a string containing
--]]
function writeToString(arr)
	local str = "";
	for k, v in pairs(arr) do
		str = ("%sL[\"%s\"] = true;\n"):format(str, tostring(k));
	end
	return str;
end

local basePattern = '%-%-@locale%-replace@';
local pattern = 'L%[\"([a-zA-Z0-9 ]+)\"%]';

local keys = {};
extractData('./Frame.lua', pattern, keys);
extractData('./Config.lua', pattern, keys);
extractData('./Main.lua', pattern, keys);

local file = io.open('./Locale.Base.lua', 'r');

local str = file:read("*all");

str = string.gsub(str, basePattern, writeToString(keys));

file:close();

file = io.open('./Locale.lua', 'w');

file:write(str);

file:close();

--[[
	Open 'Locale.Base.lua'
	Replace `basePattern` with string from writeToString(keys)
	Write new string to Locale.lua
--]]